@@ -1,7 +1,10 @@
+from __future__ import division
+
 import pickle
 from io import BytesIO
 import numpy as np
 import scipy.sparse
+import pytest
 
 from sklearn.datasets import load_digits, load_iris
 
@@ -18,7 +21,8 @@ from sklearn.utils.testing import assert_raise_message
 from sklearn.utils.testing import assert_greater
 from sklearn.utils.testing import assert_warns
 
-from sklearn.naive_bayes import GaussianNB, BernoulliNB, MultinomialNB
+from sklearn.naive_bayes import GaussianNB, BernoulliNB
+from sklearn.naive_bayes import MultinomialNB, ComplementNB
 
 # Data is just 6 separable points in the plane
 X = np.array([[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]])
@@ -108,7 +112,19 @@ def test_gnb_priors():
     assert_array_almost_equal(clf.predict_proba([[-0.1, -0.1]]),
                               np.array([[0.825303662161683,
                                          0.174696337838317]]), 8)
-    assert_array_equal(clf.class_prior_, np.array([0.3, 0.7]))
+    assert_array_almost_equal(clf.class_prior_, np.array([0.3, 0.7]))
+
+
+def test_gnb_priors_sum_isclose():
+    # test whether the class prior sum is properly tested"""
+    X = np.array([[-1, -1], [-2, -1], [-3, -2], [-4, -5], [-5, -4],
+                 [1, 1], [2, 1], [3, 2], [4, 4], [5, 5]])
+    priors = np.array([0.08, 0.14, 0.03, 0.16, 0.11, 0.16, 0.07, 0.14,
+                       0.11, 0.0])
+    Y = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
+    clf = GaussianNB(priors)
+    # smoke test for issue #9633
+    clf.fit(X, Y)
 
 
 def test_gnb_wrong_nb_priors():
@@ -162,51 +178,56 @@ def test_discrete_prior():
                                   clf.class_log_prior_, 8)
 
 
-def test_mnnb():
+@pytest.mark.parametrize('kind', ('dense', 'sparse'))
+def test_mnnb(kind):
     # Test Multinomial Naive Bayes classification.
     # This checks that MultinomialNB implements fit and predict and returns
     # correct values for a simple toy dataset.
 
-    for X in [X2, scipy.sparse.csr_matrix(X2)]:
-        # Check the ability to predict the learning set.
-        clf = MultinomialNB()
-        assert_raises(ValueError, clf.fit, -X, y2)
-        y_pred = clf.fit(X, y2).predict(X)
+    if kind == 'dense':
+        X = X2
+    elif kind == 'sparse':
+        X = scipy.sparse.csr_matrix(X2)
 
-        assert_array_equal(y_pred, y2)
+    # Check the ability to predict the learning set.
+    clf = MultinomialNB()
+    assert_raises(ValueError, clf.fit, -X, y2)
+    y_pred = clf.fit(X, y2).predict(X)
 
-        # Verify that np.log(clf.predict_proba(X)) gives the same results as
-        # clf.predict_log_proba(X)
-        y_pred_proba = clf.predict_proba(X)
-        y_pred_log_proba = clf.predict_log_proba(X)
-        assert_array_almost_equal(np.log(y_pred_proba), y_pred_log_proba, 8)
+    assert_array_equal(y_pred, y2)
 
-        # Check that incremental fitting yields the same results
-        clf2 = MultinomialNB()
-        clf2.partial_fit(X[:2], y2[:2], classes=np.unique(y2))
-        clf2.partial_fit(X[2:5], y2[2:5])
-        clf2.partial_fit(X[5:], y2[5:])
+    # Verify that np.log(clf.predict_proba(X)) gives the same results as
+    # clf.predict_log_proba(X)
+    y_pred_proba = clf.predict_proba(X)
+    y_pred_log_proba = clf.predict_log_proba(X)
+    assert_array_almost_equal(np.log(y_pred_proba), y_pred_log_proba, 8)
 
-        y_pred2 = clf2.predict(X)
-        assert_array_equal(y_pred2, y2)
+    # Check that incremental fitting yields the same results
+    clf2 = MultinomialNB()
+    clf2.partial_fit(X[:2], y2[:2], classes=np.unique(y2))
+    clf2.partial_fit(X[2:5], y2[2:5])
+    clf2.partial_fit(X[5:], y2[5:])
 
-        y_pred_proba2 = clf2.predict_proba(X)
-        y_pred_log_proba2 = clf2.predict_log_proba(X)
-        assert_array_almost_equal(np.log(y_pred_proba2), y_pred_log_proba2, 8)
-        assert_array_almost_equal(y_pred_proba2, y_pred_proba)
-        assert_array_almost_equal(y_pred_log_proba2, y_pred_log_proba)
+    y_pred2 = clf2.predict(X)
+    assert_array_equal(y_pred2, y2)
 
-        # Partial fit on the whole data at once should be the same as fit too
-        clf3 = MultinomialNB()
-        clf3.partial_fit(X, y2, classes=np.unique(y2))
+    y_pred_proba2 = clf2.predict_proba(X)
+    y_pred_log_proba2 = clf2.predict_log_proba(X)
+    assert_array_almost_equal(np.log(y_pred_proba2), y_pred_log_proba2, 8)
+    assert_array_almost_equal(y_pred_proba2, y_pred_proba)
+    assert_array_almost_equal(y_pred_log_proba2, y_pred_log_proba)
 
-        y_pred3 = clf3.predict(X)
-        assert_array_equal(y_pred3, y2)
-        y_pred_proba3 = clf3.predict_proba(X)
-        y_pred_log_proba3 = clf3.predict_log_proba(X)
-        assert_array_almost_equal(np.log(y_pred_proba3), y_pred_log_proba3, 8)
-        assert_array_almost_equal(y_pred_proba3, y_pred_proba)
-        assert_array_almost_equal(y_pred_log_proba3, y_pred_log_proba)
+    # Partial fit on the whole data at once should be the same as fit too
+    clf3 = MultinomialNB()
+    clf3.partial_fit(X, y2, classes=np.unique(y2))
+
+    y_pred3 = clf3.predict(X)
+    assert_array_equal(y_pred3, y2)
+    y_pred_proba3 = clf3.predict_proba(X)
+    y_pred_log_proba3 = clf3.predict_log_proba(X)
+    assert_array_almost_equal(np.log(y_pred_proba3), y_pred_log_proba3, 8)
+    assert_array_almost_equal(y_pred_proba3, y_pred_proba)
+    assert_array_almost_equal(y_pred_log_proba3, y_pred_log_proba)
 
 
 def check_partial_fit(cls):
@@ -225,9 +246,9 @@ def check_partial_fit(cls):
     assert_array_equal(clf1.feature_count_, clf3.feature_count_)
 
 
-def test_discretenb_partial_fit():
-    for cls in [MultinomialNB, BernoulliNB]:
-        yield check_partial_fit, cls
+@pytest.mark.parametrize("cls", [MultinomialNB, BernoulliNB])
+def test_discretenb_partial_fit(cls):
+    check_partial_fit(cls)
 
 
 def test_gnb_partial_fit():
@@ -244,62 +265,63 @@ def test_gnb_partial_fit():
     assert_array_almost_equal(clf.class_prior_, clf_pf2.class_prior_)
 
 
-def test_discretenb_pickle():
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, GaussianNB])
+def test_discretenb_pickle(cls):
     # Test picklability of discrete naive Bayes classifiers
 
-    for cls in [BernoulliNB, MultinomialNB, GaussianNB]:
-        clf = cls().fit(X2, y2)
-        y_pred = clf.predict(X2)
+    clf = cls().fit(X2, y2)
+    y_pred = clf.predict(X2)
 
+    store = BytesIO()
+    pickle.dump(clf, store)
+    clf = pickle.load(BytesIO(store.getvalue()))
+
+    assert_array_equal(y_pred, clf.predict(X2))
+
+    if cls is not GaussianNB:
+        # TODO re-enable me when partial_fit is implemented for GaussianNB
+
+        # Test pickling of estimator trained with partial_fit
+        clf2 = cls().partial_fit(X2[:3], y2[:3], classes=np.unique(y2))
+        clf2.partial_fit(X2[3:], y2[3:])
         store = BytesIO()
-        pickle.dump(clf, store)
-        clf = pickle.load(BytesIO(store.getvalue()))
-
-        assert_array_equal(y_pred, clf.predict(X2))
-
-        if cls is not GaussianNB:
-            # TODO re-enable me when partial_fit is implemented for GaussianNB
-
-            # Test pickling of estimator trained with partial_fit
-            clf2 = cls().partial_fit(X2[:3], y2[:3], classes=np.unique(y2))
-            clf2.partial_fit(X2[3:], y2[3:])
-            store = BytesIO()
-            pickle.dump(clf2, store)
-            clf2 = pickle.load(BytesIO(store.getvalue()))
-            assert_array_equal(y_pred, clf2.predict(X2))
+        pickle.dump(clf2, store)
+        clf2 = pickle.load(BytesIO(store.getvalue()))
+        assert_array_equal(y_pred, clf2.predict(X2))
 
 
-def test_input_check_fit():
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, GaussianNB])
+def test_input_check_fit(cls):
     # Test input checks for the fit method
-    for cls in [BernoulliNB, MultinomialNB, GaussianNB]:
-        # check shape consistency for number of samples at fit time
-        assert_raises(ValueError, cls().fit, X2, y2[:-1])
 
-        # check shape consistency for number of input features at predict time
-        clf = cls().fit(X2, y2)
-        assert_raises(ValueError, clf.predict, X2[:, :-1])
+    # check shape consistency for number of samples at fit time
+    assert_raises(ValueError, cls().fit, X2, y2[:-1])
+
+    # check shape consistency for number of input features at predict time
+    clf = cls().fit(X2, y2)
+    assert_raises(ValueError, clf.predict, X2[:, :-1])
 
 
-def test_input_check_partial_fit():
-    for cls in [BernoulliNB, MultinomialNB]:
-        # check shape consistency
-        assert_raises(ValueError, cls().partial_fit, X2, y2[:-1],
-                      classes=np.unique(y2))
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
+def test_input_check_partial_fit(cls):
+    # check shape consistency
+    assert_raises(ValueError, cls().partial_fit, X2, y2[:-1],
+                  classes=np.unique(y2))
 
-        # classes is required for first call to partial fit
-        assert_raises(ValueError, cls().partial_fit, X2, y2)
+    # classes is required for first call to partial fit
+    assert_raises(ValueError, cls().partial_fit, X2, y2)
 
-        # check consistency of consecutive classes values
-        clf = cls()
-        clf.partial_fit(X2, y2, classes=np.unique(y2))
-        assert_raises(ValueError, clf.partial_fit, X2, y2,
-                      classes=np.arange(42))
+    # check consistency of consecutive classes values
+    clf = cls()
+    clf.partial_fit(X2, y2, classes=np.unique(y2))
+    assert_raises(ValueError, clf.partial_fit, X2, y2,
+                  classes=np.arange(42))
 
-        # check consistency of input shape for partial_fit
-        assert_raises(ValueError, clf.partial_fit, X2[:, :-1], y2)
+    # check consistency of input shape for partial_fit
+    assert_raises(ValueError, clf.partial_fit, X2[:, :-1], y2)
 
-        # check consistency of input shape for predict
-        assert_raises(ValueError, clf.predict, X2[:, :-1])
+    # check consistency of input shape for predict
+    assert_raises(ValueError, clf.predict, X2[:, :-1])
 
 
 def test_discretenb_predict_proba():
@@ -333,34 +355,35 @@ def test_discretenb_predict_proba():
         assert_almost_equal(np.sum(np.exp(clf.intercept_)), 1)
 
 
-def test_discretenb_uniform_prior():
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
+def test_discretenb_uniform_prior(cls):
     # Test whether discrete NB classes fit a uniform prior
     # when fit_prior=False and class_prior=None
 
-    for cls in [BernoulliNB, MultinomialNB]:
-        clf = cls()
-        clf.set_params(fit_prior=False)
-        clf.fit([[0], [0], [1]], [0, 0, 1])
-        prior = np.exp(clf.class_log_prior_)
-        assert_array_equal(prior, np.array([.5, .5]))
+    clf = cls()
+    clf.set_params(fit_prior=False)
+    clf.fit([[0], [0], [1]], [0, 0, 1])
+    prior = np.exp(clf.class_log_prior_)
+    assert_array_almost_equal(prior, np.array([.5, .5]))
 
 
-def test_discretenb_provide_prior():
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
+def test_discretenb_provide_prior(cls):
     # Test whether discrete NB classes use provided prior
 
-    for cls in [BernoulliNB, MultinomialNB]:
-        clf = cls(class_prior=[0.5, 0.5])
-        clf.fit([[0], [0], [1]], [0, 0, 1])
-        prior = np.exp(clf.class_log_prior_)
-        assert_array_equal(prior, np.array([.5, .5]))
+    clf = cls(class_prior=[0.5, 0.5])
+    clf.fit([[0], [0], [1]], [0, 0, 1])
+    prior = np.exp(clf.class_log_prior_)
+    assert_array_almost_equal(prior, np.array([.5, .5]))
 
-        # Inconsistent number of classes with prior
-        assert_raises(ValueError, clf.fit, [[0], [1], [2]], [0, 1, 2])
-        assert_raises(ValueError, clf.partial_fit, [[0], [1]], [0, 1],
-                      classes=[0, 1, 1])
+    # Inconsistent number of classes with prior
+    assert_raises(ValueError, clf.fit, [[0], [1], [2]], [0, 1, 2])
+    assert_raises(ValueError, clf.partial_fit, [[0], [1]], [0, 1],
+                  classes=[0, 1, 1])
 
 
-def test_discretenb_provide_prior_with_partial_fit():
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
+def test_discretenb_provide_prior_with_partial_fit(cls):
     # Test whether discrete NB classes use provided prior
     # when using partial_fit
 
@@ -368,22 +391,21 @@ def test_discretenb_provide_prior_with_partial_fit():
     iris_data1, iris_data2, iris_target1, iris_target2 = train_test_split(
         iris.data, iris.target, test_size=0.4, random_state=415)
 
-    for cls in [BernoulliNB, MultinomialNB]:
-        for prior in [None, [0.3, 0.3, 0.4]]:
-            clf_full = cls(class_prior=prior)
-            clf_full.fit(iris.data, iris.target)
-            clf_partial = cls(class_prior=prior)
-            clf_partial.partial_fit(iris_data1, iris_target1,
-                                    classes=[0, 1, 2])
-            clf_partial.partial_fit(iris_data2, iris_target2)
-            assert_array_almost_equal(clf_full.class_log_prior_,
-                                      clf_partial.class_log_prior_)
+    for prior in [None, [0.3, 0.3, 0.4]]:
+        clf_full = cls(class_prior=prior)
+        clf_full.fit(iris.data, iris.target)
+        clf_partial = cls(class_prior=prior)
+        clf_partial.partial_fit(iris_data1, iris_target1,
+                                classes=[0, 1, 2])
+        clf_partial.partial_fit(iris_data2, iris_target2)
+        assert_array_almost_equal(clf_full.class_log_prior_,
+                                  clf_partial.class_log_prior_)
 
 
-def test_sample_weight_multiclass():
-    for cls in [BernoulliNB, MultinomialNB]:
-        # check shape consistency for number of samples at fit time
-        yield check_sample_weight_multiclass, cls
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
+def test_sample_weight_multiclass(cls):
+    # check shape consistency for number of samples at fit time
+    check_sample_weight_multiclass(cls)
 
 
 def check_sample_weight_multiclass(cls):
@@ -458,6 +480,9 @@ def test_check_accuracy_on_digits():
     scores = cross_val_score(GaussianNB(), X, y, cv=10)
     assert_greater(scores.mean(), 0.77)
 
+    scores = cross_val_score(GaussianNB(var_smoothing=0.1), X, y, cv=10)
+    assert_greater(scores.mean(), 0.89)
+
     scores = cross_val_score(GaussianNB(), X_3v8, y_3v8, cv=10)
     assert_greater(scores.mean(), 0.86)
 
@@ -467,7 +492,7 @@ def test_feature_log_prob_bnb():
     # Tests that the feature log prob value computed by BernoulliNB when
     # alpha=1.0 is equal to the expression given in Manning, Raghavan,
     # and Schuetze's "Introduction to Information Retrieval" book:
-    # http://nlp.stanford.edu/IR-book/html/htmledition/the-bernoulli-model-1.html
+    # https://nlp.stanford.edu/IR-book/html/htmledition/the-bernoulli-model-1.html
 
     X = np.array([[0, 0, 0], [1, 1, 0], [0, 1, 0], [1, 0, 1], [0, 1, 0]])
     Y = np.array([0, 0, 1, 2, 2])
@@ -489,7 +514,7 @@ def test_bnb():
     # Tests that BernoulliNB when alpha=1.0 gives the same values as
     # those given for the toy example in Manning, Raghavan, and
     # Schuetze's "Introduction to Information Retrieval" book:
-    # http://nlp.stanford.edu/IR-book/html/htmledition/the-bernoulli-model-1.html
+    # https://nlp.stanford.edu/IR-book/html/htmledition/the-bernoulli-model-1.html
 
     # Training data points are:
     # Chinese Beijing Chinese (class: China)
@@ -530,6 +555,72 @@ def test_bnb():
     assert_array_almost_equal(clf.predict_proba(X_test), predict_proba)
 
 
+def test_cnb():
+    # Tests ComplementNB when alpha=1.0 for the toy example in Manning,
+    # Raghavan, and Schuetze's "Introduction to Information Retrieval" book:
+    # https://nlp.stanford.edu/IR-book/html/htmledition/the-bernoulli-model-1.html
+
+    # Training data points are:
+    # Chinese Beijing Chinese (class: China)
+    # Chinese Chinese Shanghai (class: China)
+    # Chinese Macao (class: China)
+    # Tokyo Japan Chinese (class: Japan)
+
+    # Features are Beijing, Chinese, Japan, Macao, Shanghai, and Tokyo.
+    X = np.array([[1, 1, 0, 0, 0, 0],
+                  [0, 1, 0, 0, 1, 0],
+                  [0, 1, 0, 1, 0, 0],
+                  [0, 1, 1, 0, 0, 1]])
+
+    # Classes are China (0), Japan (1).
+    Y = np.array([0, 0, 0, 1])
+
+    # Check that weights are correct. See steps 4-6 in Table 4 of
+    # Rennie et al. (2003).
+    theta = np.array([
+        [
+            (0 + 1) / (3 + 6),
+            (1 + 1) / (3 + 6),
+            (1 + 1) / (3 + 6),
+            (0 + 1) / (3 + 6),
+            (0 + 1) / (3 + 6),
+            (1 + 1) / (3 + 6)
+        ],
+        [
+            (1 + 1) / (6 + 6),
+            (3 + 1) / (6 + 6),
+            (0 + 1) / (6 + 6),
+            (1 + 1) / (6 + 6),
+            (1 + 1) / (6 + 6),
+            (0 + 1) / (6 + 6)
+        ]])
+
+    weights = np.zeros(theta.shape)
+    normed_weights = np.zeros(theta.shape)
+    for i in range(2):
+        weights[i] = -np.log(theta[i])
+        normed_weights[i] = weights[i] / weights[i].sum()
+
+    # Verify inputs are nonnegative.
+    clf = ComplementNB(alpha=1.0)
+    assert_raises(ValueError, clf.fit, -X, Y)
+
+    clf.fit(X, Y)
+
+    # Check that counts/weights are correct.
+    feature_count = np.array([[1, 3, 0, 1, 1, 0], [0, 1, 1, 0, 0, 1]])
+    assert_array_equal(clf.feature_count_, feature_count)
+    class_count = np.array([3, 1])
+    assert_array_equal(clf.class_count_, class_count)
+    feature_all = np.array([1, 4, 1, 1, 1, 1])
+    assert_array_equal(clf.feature_all_, feature_all)
+    assert_array_almost_equal(clf.feature_log_prob_, weights)
+
+    clf = ComplementNB(alpha=1.0, norm=True)
+    clf.fit(X, Y)
+    assert_array_almost_equal(clf.feature_log_prob_, normed_weights)
+
+
 def test_naive_bayes_scale_invariance():
     # Scaling the data should not change the prediction results
     iris = load_iris()
@@ -584,3 +675,45 @@ def test_alpha():
                          X, y, classes=[0, 1])
     assert_raise_message(ValueError, expected_msg, m_nb.partial_fit,
                          X, y, classes=[0, 1])
+
+
+def test_alpha_vector():
+    X = np.array([[1, 0], [1, 1]])
+    y = np.array([0, 1])
+
+    # Setting alpha=np.array with same length
+    # as number of features should be fine
+    alpha = np.array([1, 2])
+    nb = MultinomialNB(alpha=alpha)
+    nb.partial_fit(X, y, classes=[0, 1])
+
+    # Test feature probabilities uses pseudo-counts (alpha)
+    feature_prob = np.array([[1 / 2, 1 / 2], [2 / 5, 3 / 5]])
+    assert_array_almost_equal(nb.feature_log_prob_, np.log(feature_prob))
+
+    # Test predictions
+    prob = np.array([[5 / 9, 4 / 9], [25 / 49, 24 / 49]])
+    assert_array_almost_equal(nb.predict_proba(X), prob)
+
+    # Test alpha non-negative
+    alpha = np.array([1., -0.1])
+    expected_msg = ('Smoothing parameter alpha = -1.0e-01. '
+                    'alpha should be > 0.')
+    m_nb = MultinomialNB(alpha=alpha)
+    assert_raise_message(ValueError, expected_msg, m_nb.fit, X, y)
+
+    # Test that too small pseudo-counts are replaced
+    ALPHA_MIN = 1e-10
+    alpha = np.array([ALPHA_MIN / 2, 0.5])
+    m_nb = MultinomialNB(alpha=alpha)
+    m_nb.partial_fit(X, y, classes=[0, 1])
+    assert_array_almost_equal(m_nb._check_alpha(),
+                              [ALPHA_MIN, 0.5],
+                              decimal=12)
+
+    # Test correct dimensions
+    alpha = np.array([1., 2., 3.])
+    m_nb = MultinomialNB(alpha=alpha)
+    expected_msg = ('alpha should be a scalar or a numpy array '
+                    'with shape [n_features]')
+    assert_raise_message(ValueError, expected_msg, m_nb.fit, X, y)
